/* tcp.h
 * Copyright (C) 2005-2006 Tillmann Werner <tillmann.werner@gmx.de>
 *
 * This file is free software; as a special exception the author gives
 * unlimited permission to copy and/or distribute it, with or without
 * modifications, as long as this notice is preserved.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef __HONEYTRAP_TCP_H
#define __HONEYTRAP_TCP_H 1


/* tcp header */
struct tcp_header{
	uint16_t	th_sport;	/* tcp source port */
	uint16_t	th_dport;	/* tcp dest port */
	uint32_t	th_seqno;	/* tcp sequence number,identifies the byte in the stream of data */
	uint32_t	th_ackno;	/* contains the next seq num that the sender expects to recieve */
	u_char		th_res:4,	/* 4 reserved bits */
			th_doff:4;	/* data offset */
	u_char		th_flags;	/* tcp flags */
	uint16_t	th_window;	/* maxinum number of bytes able to recieve*/
	uint16_t	th_sum;		/* checksum to cover the tcp header and data portion of the packet*/
	uint16_t	th_urp;		/* vaild only if the urgent flag is set, used to transmit emergency data */
};

#define FIN 0x01
#define SYN 0x02
#define RST 0x04
#define PUSH 0x08
#define ACK 0x10
#define URG 0x20
#define ECE 0x40
#define CWR 0x80
#define FLAGS (TH_FIN|TH_SYN|TH_RST|TH_ACK|TH_URG|TH_ECE|TH_CWR)

const struct tcp_header *tcp;


int tcpsock(struct sockaddr_in *server_addr, uint16_t port);
int tcpcopy(int to_fd, int from_fd, u_char **save_string, uint32_t offset, int *bytes_read, int *bytes_sent);

#endif
